package com.glxp.udidl.admin.service.udplat.impl;

import com.alibaba.fastjson.JSON;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.glxp.udidl.admin.dao.udplat.UdplatGoodsMapper;
import com.glxp.udidl.admin.dto.udplat.BaseParam;
import com.glxp.udidl.admin.dto.udplat.DeliveryGoods;
import com.glxp.udidl.admin.dto.udplat.HospitalGoods;
import com.glxp.udidl.admin.entity.udplat.UdplatGoods;
import com.glxp.udidl.admin.entity.udplat.UdplatLog;
import com.glxp.udidl.admin.res.PageSimpleResponse;
import com.glxp.udidl.admin.service.udplat.UdplatGoodsService;
import com.glxp.udidl.admin.service.udplat.UdplatLogService;
import com.glxp.udidl.admin.util.BeanUtils;
import com.glxp.udidl.admin.util.DateUtil;
import com.glxp.udidl.common.res.BaseResponse;
import com.glxp.udidl.common.util.ExcelUtil;
import com.glxp.udidl.common.util.ResultVOUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.InputStream;
import java.util.Date;
import java.util.List;

@Service
@Slf4j
public class UdplatGoodsServiceImpl implements UdplatGoodsService {
    @Autowired
    private UdplatGoodsMapper udplatGoodsMapper;
    @Autowired
    private UdplatLogService logService;

    public BaseResponse getList(BaseParam param){
        PageHelper.startPage(param.getPage(), param.getLimit());
        if (param.getEndDate() != null && !param.getEndDate().isEmpty())
            param.setEndDate(DateUtil.formatDate(DateUtil.addDays(DateUtil.parseDate(param.getEndDate()), 1)));
        List<UdplatGoods> list = udplatGoodsMapper.list(param);
        PageInfo<UdplatGoods> pageInfo = new PageInfo<>(list);
        PageSimpleResponse<UdplatGoods> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(list);
        return ResultVOUtils.success(pageSimpleResponse);
    }
    @Override
    public BaseResponse importExcel(InputStream is, int type) {
        int count=0;
        try{
            if(type == 1) {
                List<HospitalGoods> list1 = getData1(is);
                //list = BeanUtils.convertList2List(list1,UdplatGoods.class);
                if(list1 == null || list1.size()<1)
                    return ResultVOUtils.error(-1,"无数据！");
                count = SaveHospitalGoods(list1);
            }else
            if(type == 2){
                List<DeliveryGoods> list2 = getData2(is);
                //log.info(JSONUtil.toJsonStr(list2));
                //list = BeanUtils.convertList2List(list2,UdplatGoods.class);
                if(list2 == null || list2.size()<1)
                    return ResultVOUtils.error(-1,"无数据！");
                count = SaveDeliveryGoods(list2);
            }else
                return  ResultVOUtils.error(-1,"文件类型不匹配！");
            UdplatLog udplatLog = new UdplatLog();
            udplatLog.setDownloadType("import");
            udplatLog.setTbName("udplat_goods");
            udplatLog.setType("info");
            udplatLog.setMsg("执行成功！");
            udplatLog.setActionDate(DateUtil.parseDate(DateUtil.getDate()));
            udplatLog.setTotalCount(count);
            udplatLog.setCreateTime(new Date());
            logService.insert(udplatLog);
            return ResultVOUtils.success(count);
            //return save(list);
        }catch (Exception e){
            UdplatLog udplatLog = new UdplatLog();
            udplatLog.setDownloadType("import");
            udplatLog.setTbName("udplat_goods");
            udplatLog.setType("error");
            udplatLog.setMsg(e.getMessage());
            udplatLog.setContent(JSON.toJSONString(e.getStackTrace()));
            udplatLog.setActionDate(DateUtil.parseDate(DateUtil.getDate()));
            udplatLog.setCreateTime(new Date());
            logService.insert(udplatLog);
            return ResultVOUtils.error(-1,"出错了:"+e.getMessage());
        }

        //return ResultVOUtils.success();
    }
    private List<HospitalGoods> getData1(InputStream is) throws Exception{
        ExcelUtil<HospitalGoods> util = new ExcelUtil(HospitalGoods.class);
        return util.importExcel(is);
    }
    private List<DeliveryGoods> getData2(InputStream is) throws Exception{
        ExcelUtil<DeliveryGoods> util = new ExcelUtil(DeliveryGoods.class);
        return util.importExcel(is);
    }
    private int SaveHospitalGoods(List<HospitalGoods> list){
        int count=0;
        for(HospitalGoods item:list){
            String goodId = item.getDeliveryGoodId();
            if(goodId == null || goodId.isEmpty())
                continue;
            UdplatGoods goods = udplatGoodsMapper.selectByGoodId(goodId);
            goods = HospitalGoodsToUdplatGoods(item,goods);
            if(goods.getId()>0){
                goods.setUpdateTime(new Date());
                udplatGoodsMapper.update(goods);
            }else
            {
                goods.setUuid(BeanUtils.getUUId());
                goods.setCreateTime(new Date());
                udplatGoodsMapper.insert(goods);
            }
            count++;
        }
        return count;
    }
    private int SaveDeliveryGoods(List<DeliveryGoods> list){
        int count=0;
        for(DeliveryGoods item:list){
            String goodId = item.getDeliveryGoodId();
            if(goodId == null || goodId.isEmpty())
                continue;
            UdplatGoods goods = udplatGoodsMapper.selectByGoodId(goodId);
            goods = DeliveryGoodsToUdplatGoods(item,goods);
            System.out.println(JSON.toJSONString(goods));
            if(goods.getId()>0L){
                goods.setUpdateTime(new Date());
                udplatGoodsMapper.update(goods);
            }else
            {
                goods.setUuid(BeanUtils.getUUId());
                goods.setCreateTime(new Date());
                udplatGoodsMapper.insert(goods);
            }
            count++;
        }
        return count;
    }
    private UdplatGoods HospitalGoodsToUdplatGoods(HospitalGoods hospitalGoods, UdplatGoods udplatGoods){
        if(udplatGoods == null){
            udplatGoods = new UdplatGoods();
            udplatGoods.setId(0L);
        }
        udplatGoods.setDeliveryGoodId(hospitalGoods.getDeliveryGoodId());
        udplatGoods.setManufactureName(hospitalGoods.getManufactureName());
        udplatGoods.setDistributorName(hospitalGoods.getDistributorName());
        udplatGoods.setProductName(hospitalGoods.getProductName());
        udplatGoods.setModel(hospitalGoods.getModel());
        udplatGoods.setSpec(hospitalGoods.getSpec());
        udplatGoods.setProdMaterial(hospitalGoods.getProdMaterial());
        udplatGoods.setRegNum(hospitalGoods.getRegNum());
        udplatGoods.setRegName(hospitalGoods.getRegName());
        udplatGoods.setRegValidTo(hospitalGoods.getRegValidTo());
        udplatGoods.setSalePrice(hospitalGoods.getSalePrice());
        udplatGoods.setPriceUnitText(hospitalGoods.getPriceUnitText());
        udplatGoods.setChangedContent(hospitalGoods.getChangedContent());
        udplatGoods.setChangedTime(hospitalGoods.getChangedTime());
        udplatGoods.setMedicalCode(hospitalGoods.getMedicalCode());
        return udplatGoods;
    }
    private UdplatGoods DeliveryGoodsToUdplatGoods(DeliveryGoods deliveryGoods, UdplatGoods udplatGoods){
        if(udplatGoods == null)
        {
            udplatGoods = new UdplatGoods();
            udplatGoods.setId(0L);
        }
        udplatGoods.setDeliveryGoodId(deliveryGoods.getDeliveryGoodId());
        udplatGoods.setProductName(deliveryGoods.getProductName());
        udplatGoods.setModel(deliveryGoods.getModel());
        udplatGoods.setSpec(deliveryGoods.getSpec());
        udplatGoods.setProdMaterial(deliveryGoods.getProdMaterial());
        udplatGoods.setPackMaterial(deliveryGoods.getPackMaterial());
        udplatGoods.setRegNum(deliveryGoods.getRegNum());
        udplatGoods.setRegName(deliveryGoods.getRegName());
        udplatGoods.setRegValidTo(deliveryGoods.getRegValidTo());
        udplatGoods.setStockStatus(deliveryGoods.getStockStatus());
        udplatGoods.setChangedContent(deliveryGoods.getChangedContent());
        udplatGoods.setChangedTime(deliveryGoods.getChangedTime());
        udplatGoods.setMedicalCode(deliveryGoods.getMedicalCode());
        return udplatGoods;
    }
    /*private BaseResponse save(List<UdplatGoods> list){
        if(list == null || list.size()<1)
            return ResultVOUtils.error(-1,"无数据！");
        int count=0;
        for(UdplatGoods item:list){
            String goodId = item.getDeliveryGoodId();
            if(goodId == null || goodId.isEmpty())
                continue;
            UdplatGoods goods = udplatGoodsMapper.selectByGoodId(goodId);
            if(goods != null)
            {
               BeanUtils.copyProperties(item,goods);
               goods.setUpdateTime(new Date());
               udplatGoodsMapper.update(goods);
            }else{
                item.setUuid(BeanUtils.getUUId());
                item.setCreateTime(new Date());
                udplatGoodsMapper.insert(item);
            }
            count++;
        }
        UdplatLog udplatLog = new UdplatLog();
        udplatLog.setDownloadType("import");
        udplatLog.setTbName("udplat_goods");
        udplatLog.setType("info");
        udplatLog.setMsg("执行成功！");
        udplatLog.setActionDate(DateUtil.parseDate(DateUtil.getDate()));
        udplatLog.setTotalCount(count);
        udplatLog.setCreateTime(new Date());
        logService.insert(udplatLog);
        return ResultVOUtils.success(count);
    }*/
}
